@@ -5,7 +5,6 @@ use crate::handler::HandlerOutput;
 use crate::ics04_channel::handler::send_packet::send_packet;
 use crate::ics04_channel::packet::Packet;
 use crate::ics04_channel::packet::PacketResult;
-use crate::timestamp::Timestamp;
 
 pub(crate) fn send_transfer<Ctx>(
     ctx: &Ctx,
@@ -35,9 +34,6 @@ where
             Kind::SequenceSendNotFound(msg.source_port.clone(), msg.source_channel.clone())
         })?;
 
-    let timeout_timestamp = Timestamp::from_nanoseconds(msg.timeout_timestamp.as_nanoseconds())
-        .map_err(|_| Kind::InvalidPacketTimeoutTimestamp(msg.timeout_timestamp.as_nanoseconds()))?;
-
     //TODO: Application LOGIC.
 
     let packet = Packet {
@@ -48,7 +44,7 @@ where
         destination_channel: destination_channel.clone(),
         data: vec![0],
         timeout_height: msg.timeout_height,
-        timeout_timestamp,
+        timeout_timestamp: msg.timeout_timestamp,
     };
 
     let handler_output =
