@@ -6,8 +6,10 @@ use serde_derive::{Deserialize, Serialize};
 use crate::ics02_client::events as ClientEvents;
 use crate::ics02_client::events::NewBlock;
 use crate::ics03_connection::events as ConnectionEvents;
+use crate::ics03_connection::events::Attributes as ConnectionAttributes;
 use crate::ics04_channel::events as ChannelEvents;
 use crate::ics04_channel::events::Attributes as ChannelAttributes;
+
 use crate::Height;
 use prost::alloc::fmt::Formatter;
 use std::fmt;
@@ -197,6 +199,15 @@ impl IbcEvent {
             _ => None,
         }
     }
+    pub fn connection_attributes(&self) -> Option<&ConnectionAttributes> {
+        match self {
+            IbcEvent::OpenInitConnection(ev) => Some(ev.attributes()),
+            IbcEvent::OpenTryConnection(ev) => Some(ev.attributes()),
+            IbcEvent::OpenAckConnection(ev) => Some(ev.attributes()),
+            IbcEvent::OpenConfirmConnection(ev) => Some(ev.attributes()),
+            _ => None,
+        }
+    }
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize)]
